@@ -28,7 +28,6 @@ from _pytest.outcomes import skip
 from _pytest.outcomes import xfail
 from _pytest.python import Class
 from _pytest.python import Function
-from _pytest.python import Generator
 from _pytest.python import Instance
 from _pytest.python import Module
 from _pytest.python import Package
@@ -36,8 +35,14 @@ from _pytest.python_api import approx
 from _pytest.python_api import raises
 from _pytest.recwarn import deprecated_call
 from _pytest.recwarn import warns
+from _pytest.warning_types import PytestAssertRewriteWarning
+from _pytest.warning_types import PytestCacheWarning
+from _pytest.warning_types import PytestCollectionWarning
+from _pytest.warning_types import PytestConfigWarning
 from _pytest.warning_types import PytestDeprecationWarning
 from _pytest.warning_types import PytestExperimentalApiWarning
+from _pytest.warning_types import PytestUnhandledCoroutineWarning
+from _pytest.warning_types import PytestUnknownMarkWarning
 from _pytest.warning_types import PytestWarning
 from _pytest.warning_types import RemovedInPytest4Warning
 
@@ -57,7 +62,6 @@ __all__ = [
     "fixture",
     "freeze_includes",
     "Function",
-    "Generator",
     "hookimpl",
     "hookspec",
     "importorskip",
@@ -68,8 +72,14 @@ __all__ = [
     "Module",
     "Package",
     "param",
+    "PytestAssertRewriteWarning",
+    "PytestCacheWarning",
+    "PytestCollectionWarning",
+    "PytestConfigWarning",
     "PytestDeprecationWarning",
     "PytestExperimentalApiWarning",
+    "PytestUnhandledCoroutineWarning",
+    "PytestUnknownMarkWarning",
     "PytestWarning",
     "raises",
     "register_assert_rewrite",
